/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#include <malloc.h>
#include <stdint.h>

#include "codecFactory.h"

#include "base/baseCodec.h"
#include "echo/echoCodec.h"
#include "echoDSP/echoDSPCodec.h"
#include "G722DSP/g722DSPCodec.h"
#include "G722PLCDSP/g722_PLC_DSPCodec.h"
#include "celtDSP/celtDSPCodec.h"
#include "celtEncDSP/celtEncDSPCodec.h"
#include "celtDecDSP/celtDecDSPCodec.h"

/* ----------------------------------------------------------------------------
 * Function     : makeCodec
 * ----------------------------------------------------------------------------
 * Description  : factory method to create a codec of the specified type
 * Inputs       : type : the type of the codec to be created, these are
 *                defined in the KnownCodecs enum in codec.h
 * Outputs      : A handle to a codec control block which has been
 *                constructed using malloc
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
CODEC makeCodec(KnownCodecs type)
{
    if (type == BASE)
    {
        return makeBaseCodec();
    }
    if (type == ECHO)
    {
        return makeEchoCodec();
    }
    if (type == ECHO_DSP)
    {
        return makeEchoDSPCodec();
    }
    if (type == G722_DSP)
    {
        return makeG722DSPCodec();
    }
    if (type == G722_PLC_DSP)
    {
        return makeG722PLCDSPCodec();
    }
    if (type == CELT_DSP)
    {
        return makeCeltDSPCodec();
    }
    if (type == CELT_DEC_DSP)
    {
        return makeCeltDecDSPCodec();
    }
    if (type == CELT_ENC_DSP)
    {
        return makeCeltEncDSPCodec();
    }
    return NULL;
}

/* ----------------------------------------------------------------------------
 * Function     : populateCodec
 * ----------------------------------------------------------------------------
 * Description  : A factory method to populate a pre-allocated buffer with a
 *                codec control block suitable for the requested codec type.
 * Inputs       : type : the type of the codec to be created, these are
 *                defined in the KnownCodecs enum in codec.h
 *                buffer : an area of memory large enough to hold the
 *                requested codec control block
 *                size : the maximum size of the buffer, if this is not big
 *                enough to hold the codec control block the selected
 *                populate method will fail.
 * Outputs      : A handle to a codec control block which will reside in the
 *                given buffer
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
CODEC populateCodec(KnownCodecs type, unsigned char *buffer, uint32_t size)
{
    if (type == BASE)
    {
        return populateBaseCodec(buffer, size);
    }
    if (type == ECHO)
    {
        return populateEchoCodec(buffer, size);
    }
    if (type == ECHO_DSP)
    {
        return populateEchoDSPCodec(buffer, size);
    }
    if (type == G722_DSP)
    {
        return populateG722DSPCodec(buffer, size);
    }
    if (type == G722_PLC_DSP)
    {
        return populateG722PLCDSPCodec(buffer, size);
    }
    if (type == CELT_DSP)
    {
        return populateCeltDSPCodec(buffer, size);
    }
    if (type == CELT_DEC_DSP)
    {
        return populateCeltDecDSPCodec(buffer, size);
    }
    if (type == CELT_ENC_DSP)
    {
        return populateCeltEncDSPCodec(buffer, size);
    }
    return NULL;
}
