/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include <rsl10.h>

#include "logger.h"
#include "codecs/codec.h"
#include "codecs/codecFactory.h"
#include "codecs/baseDSP/baseDSPCodec.h"
#include "sharedBuffers.h"
#include "scratch.h"
#include "lpdsp32_JTAG.h"
#include "audioProcessor.h"
#include "io/echo/ioEcho.h"
#include "io/gdb/ioGDB.h"
#include "io/g722/ioG722Encode.h"
#include "io/g722/ioG722Decode.h"
#include "io/celt/ioCeltDecode.h"
#include "io/celt/ioCeltEncode.h"

extern unsigned int *ISR_Vector_Table;

/* DIO number that is used for easy re-flashing (recovery mode) */
#define RECOVERY_DIO                    12

/* ----------------------------------------------------------------------------
 * Function     : enable48MHzOscillator
 * ----------------------------------------------------------------------------
 * Description  : Helper function to set up the 48MHz clock for fast
 *                operation
 * Inputs       : None
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void enable48MHzOscillator(void)
{
    /* Configure the current trim settings for VCC, VDDA */
    ACS_VCC_CTRL->ICH_TRIM_BYTE = VCC_ICHTRIM_16MA;
    ACS_VDDA_CP_CTRL->PTRIM_BYTE = VDDA_PTRIM_16MA;

    /* Start 48 MHz XTAL oscillator */
    ACS_VDDRF_CTRL->ENABLE_ALIAS = VDDRF_ENABLE_BITBAND;
    ACS_VDDRF_CTRL->CLAMP_ALIAS = VDDRF_DISABLE_HIZ_BITBAND;

    /* Wait until VDDRF supply has powered up */
    while (ACS_VDDRF_CTRL->READY_ALIAS != VDDRF_READY_BITBAND);

    ACS_VDDPA_CTRL->ENABLE_ALIAS = VDDPA_DISABLE_BITBAND;
    ACS_VDDPA_CTRL->VDDPA_SW_CTRL_ALIAS = VDDPA_SW_VDDRF_BITBAND;

    /* Enable RF power switches */
    SYSCTRL_RF_POWER_CFG->RF_POWER_ALIAS = RF_POWER_ENABLE_BITBAND;

    /* Remove RF isolation */
    SYSCTRL_RF_ACCESS_CFG->RF_ACCESS_ALIAS = RF_ACCESS_ENABLE_BITBAND;

    /* Start the 48 MHz oscillator without changing the other register bits */
    RF->XTAL_CTRL = ((RF->XTAL_CTRL & ~XTAL_CTRL_DISABLE_OSCILLATOR) |
                     XTAL_CTRL_REG_VALUE_SEL_INTERNAL);

    /* Enable 48 MHz oscillator divider at desired prescale value */
    RF_REG2F->CK_DIV_1_6_CK_DIV_1_6_BYTE = CK_DIV_1_6_PRESCALE_2_BYTE;

    /* Wait until 48 MHz oscillator is started */
    while (RF_REG39->ANALOG_INFO_CLK_DIG_READY_ALIAS !=
           ANALOG_INFO_CLK_DIG_READY_BITBAND);

    /* Switch to (divided 48 MHz) oscillator clock */
    Sys_Clocks_SystemClkConfig(JTCK_PRESCALE_1 |
                               EXTCLK_PRESCALE_1 |
                               SYSCLK_CLKSRC_RFCLK);
}

/* ----------------------------------------------------------------------------
 * Function     : initialiseInterruptState
 * ----------------------------------------------------------------------------
 * Description  : Initialise the interrupt state and do some other housework
 *                prior to the main application starting.
 * Inputs       : None
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void initialiseInterruptState(void)
{
    SCB->VTOR = (unsigned int)(&ISR_Vector_Table);

    Sys_NVIC_DisableAllInt();
    Sys_NVIC_ClearAllPendingInt();

    SYSCTRL->DSS_CTRL = DSS_LPDSP32_PAUSE;

    /* Test DIO12 to pause the program to make it easy to re-flash */
    DIO->CFG[RECOVERY_DIO] = DIO_MODE_INPUT | DIO_WEAK_PULL_UP |
                             DIO_LPF_DISABLE | DIO_6X_DRIVE;
    while (DIO_DATA->ALIAS[RECOVERY_DIO] == 0);

    enable48MHzOscillator();

    /* straight out of the manual, set timer 0 to run */
    Sys_Timer_Set_Control(0, TIMER_FREE_RUN | TIMER_PRESCALE_32 | 400);

    NVIC_EnableIRQ(DSP0_IRQn);
    NVIC_EnableIRQ(FLASH_COPY_IRQn);
    NVIC_EnableIRQ(TIMER0_IRQn);

    __set_PRIMASK(PRIMASK_ENABLE_INTERRUPTS);
    __set_FAULTMASK(FAULTMASK_ENABLE_INTERRUPTS);

    /* run timer 0 */
    Sys_Timers_Start(SELECT_TIMER0);
}

/* ----------------------------------------------------------------------------
 * Function     : TIMER0_IRQHandler
 * ----------------------------------------------------------------------------
 * Description  : Timer 0 interrupt handler. This is currently used for
 *                debug purposes only.
 * Inputs       : None
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
void TIMER0_IRQHandler(void)
{
    /* we don't need to do anything with this, it is just to ensure
     * the system doesn't hang waiting for an interrupt from the LPDSP32
     */
    logInfo(0x05A);
}

/* ----------------------------------------------------------------------------
 * Function     : main
 * ----------------------------------------------------------------------------
 * Description  : Main function, this shows how a new codec may be
 *                integrated into a system,
 *
 *                1 create the codec control block, here we use
 *                  pre-allocated memory
 *                2 initialise the codec
 *                3 if we are using the DSP (which we are) execute the
 *                  handshake
 *                4 set up the status and output buffers
 *                5 initialise the audioProcessor which manages the flow
 *                  of samples
 *                6 run the audio processor state machine continuously,
 *                  putting the device to sleep if there is nothing to do.
 * Inputs       : None
 * Outputs      : return 0
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
int main(void)
{
    initialiseInterruptState();
    logInitialise();
    configureGPIOForJTAG();

    memset(Buffer.configuration, 0, CODEC_CONFIGURATION_SIZE);
    memset(Buffer.scratch, 0, CODEC_SCRATCH_MEMORY_SIZE);

    scratchInitialise(Buffer.scratch);

    /* Create a codec located in the configuration area. */
    /* IMPORTANT: make sure to update the shared memory address (DRAM_DSP_CM3) 
       in sections.ld if you change the CODEC type. */
    CODEC myCodec = populateCodec(G722_PLC_DSP,
                                  Buffer.configuration, CODEC_CONFIGURATION_SIZE);

    if (!codecIsCodec(myCodec))
    {
        logFatal(0xFFF);
    }

    /* initialise the codec */
    codecInitialise(myCodec);

    /* we have a handshake protocol to ensure the DSP is alive and in synch
     * before we try to use it.
     */
    dspHandshake(myCodec);

    /* Configure the static parts of the codec */
    codecSetStatusBuffer(myCodec, Buffer.configuration,
                         CODEC_CONFIGURATION_SIZE);
    codecSetOutputBuffer(myCodec, Buffer.output, CODEC_OUTPUT_SIZE);

    /* configure the incoming audio data processor and use the
     * audio processor to fetch data and feed it to the codec
     */
    audioProcessorInitialise(myCodec, makeGDBSourceSink());

    while (1)
    {
        logInfo(0x001);
        if (!audioProcess(myCodec))
        {
            SYS_WAIT_FOR_INTERRUPT;
        }
        Sys_Watchdog_Refresh();
    }

    return 0;
}
