/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * -----------------------------------------------------------------------------
 */

#include <string.h>

#include "..\wrapper\wrapper.h"

#include "opus_custom.h"
#include "arch.h"

#include "mem_func.h"

typedef struct {
	OpusCustomMode *encoderMode;
	OpusCustomEncoder *encoder;
	opus_uint32 encoderFinalRange;
} tChannelDefinition;

static tChannelDefinition channels[2];


/* ----------------------------------------------------------------------------
 * Function : configure
 * ----------------------------------------------------------------------------
 * Description : Implements the configure method defined in "wrapper.h"
 *
 *     Specifically, the method is used to create the initial mode of the codec
 *     based on the frame size, sample rate, mode and channel parameters, it then
 *     uses this to create suitable encoders and decoders for later use.
 *
 * Inputs : refer "wrapper.h"
 * Outputs : refer "wrapper.h"
 * Assumptions : refer "wrapper.h"
 * ----------------------------------------------------------------------------
 */
void configure(pBaseDSPCodec pConfig)
{

    int errorState;

    volatile OperationParameters *op = &pConfig->parent.operation;
    int frameSize = op->frameSize;
    int sampleRate = op->sampleRate;

    /* only interested in the "mode" component which specifies the number of
     * channels being used
     */
    int numChannels = op->modeAndChannel & 0x0F;
    int channel = (op->modeAndChannel >> 4) & 0x0F;

    channels[0].encoderMode = opus_custom_mode_create(sampleRate, frameSize, &errorState);

    if (channels[0].encoderMode == NULL || errorState != OPUS_OK)
    {
        pConfig->dspError = errorState;
        return;
    }

    channels[1].encoderMode = opus_custom_mode_create(sampleRate, frameSize, &errorState);

    if (channels[1].encoderMode == NULL || errorState != OPUS_OK)
    {
        pConfig->dspError = errorState;
        return;
    }

    channels[0].encoder = opus_custom_encoder_create(channels[0].encoderMode, numChannels, &errorState);

    if (errorState != OPUS_OK)
    {
        pConfig->dspError = errorState;
        return;
    }

    channels[1].encoder = opus_custom_encoder_create(channels[1].encoderMode, numChannels, &errorState);
    if (errorState != OPUS_OK)
    {
        pConfig->dspError = errorState;
        return;
    }

    opus_custom_encoder_ctl(channels[0].encoder, OPUS_SET_COMPLEXITY(0));
    opus_custom_encoder_ctl(channels[1].encoder, OPUS_SET_COMPLEXITY(0));
}

/* ----------------------------------------------------------------------------
 * Function : encodeReset
 * ----------------------------------------------------------------------------
 * Description : Implements the encodeReset method defined in "wrapper.h"
 * Inputs : refer "wrapper.h"
 * Outputs : refer "wrapper.h"
 * Assumptions : refer "wrapper.h"
 * ----------------------------------------------------------------------------
 */
void encodeReset(pBaseDSPCodec /* unused */)
{
    /* Nothing required here, not used for CELT codec */
}

/* ----------------------------------------------------------------------------
 * Function : decodeReset
 * ----------------------------------------------------------------------------
 * Description : Implements the decodeReset method defined in "wrapper.h"
 * Inputs : refer "wrapper.h"
 * Outputs : refer "wrapper.h"
 * Assumptions : refer "wrapper.h"
 * ----------------------------------------------------------------------------
 */
void decodeReset(pBaseDSPCodec /* unused */)
{
    /* Nothing required here, not used for CELT codec */
}

/* ----------------------------------------------------------------------------
 * Function : encode
 * ----------------------------------------------------------------------------
 * Description : Implements the encode method defined in "wrapper.h"
 * Inputs : refer "wrapper.h"
 * Outputs : refer "wrapper.h"
 * Assumptions : refer "wrapper.h"
 * ----------------------------------------------------------------------------
 */
void encode(pBaseDSPCodec pConfig, unsigned char *inBuffer,
        unsigned char *outBuffer)
{

    volatile OperationParameters *op = &pConfig->parent.operation;
    int frameSize = op->frameSize;
    int blockSize = op->blockSize;
    int channel = (op->modeAndChannel >> 4) & 0x0F;

    int length = opus_custom_encode(channels[channel].encoder, (opus_int16 *) inBuffer, frameSize,
            outBuffer, blockSize);
    if (length != blockSize)
    {
        pConfig->dspError = length;
        return;
    }

    opus_custom_encoder_ctl(channels[channel].encoder, OPUS_GET_FINAL_RANGE(&channels[channel].encoderFinalRange));
    pConfig->parent.outputBufferUsedSize = length;
}

/* ----------------------------------------------------------------------------
 * Function : decode
 * ----------------------------------------------------------------------------
 * Description : Implements the decode method defined in "wrapper.h"
 * Inputs : refer "wrapper.h"
 * Outputs : refer "wrapper.h"
 * Assumptions : refer "wrapper.h"
 * ----------------------------------------------------------------------------
 */
void decode(pBaseDSPCodec pConfig, unsigned char *inBuffer,
        unsigned char *outBuffer, bool /* unused */)
{
    /* Nothing required here, not used for CELT ENC codec */
}

