/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#include <string.h>
#include <stdlib.h>

#include <rsl10_lpdsp32.h>
#include <rsl10_lpdsp32_vector.h>

#include "mem_func.h"

#include "sharedBuffers.h"
#include "codecs/baseDSP/baseDSPCodecInternal.h"

#include "scratch.h"
#include "wrapper.h"

/* for now these must match the CM3 definitions so hard code them, but
 * in future this should be in the bcf file.
 */

/* Define the shared memory between the ARM and LPDSP processors */
unsigned char chess_storage(DMA:0x0000B800) sharedMemory[0x0800];

/* Define the interrupt register to notify the ARM of a completed operation */
volatile static unsigned char chess_storage(DMIO:ARM_CMD) CssCmdGen;

/* Flag indicating if there is pending data, set when the interrupt service
 * routine handling requests from the CM3 is activated
 */
static volatile int actionRequired;

/* simple monitor used to provide feedback of progress to CM3 */
static volatile uint32_t *statusLocation;

/* ----------------------------------------------------------------------------
 * Function     : ARM1_IRQHandler
 * ----------------------------------------------------------------------------
 * Description  : Interrupt service routine invoked on request from the CM3 
 * Inputs       : None 
 * Outputs      : sets flag indicating work needs to be done 
 * Assumptions  : None 
 * ------------------------------------------------------------------------- */
extern "C" void ARM1_IRQHandler(void) property (isr)
{
    /* raise the flag indicating something needs to be processed */
    actionRequired = true;
}

/* ----------------------------------------------------------------------------
 * Function     : relocateSharedAddress
 * ----------------------------------------------------------------------------
 * Description  : Converts an address in CM3 memory space to a corresponding 
 *                one in LPDSP32 (DMB) memory space. 
 * Inputs       : config - the base address of the configuration area in DMB 
 *                memory space CM3Address - the CM3 based address to be 
 *                converted 
 * Outputs      : The corresponding LPDSP32 address 
 * Assumptions  : None 
 * ------------------------------------------------------------------------- */
static unsigned char chess_storage(DMB) *relocateSharedAddress(
		struct _baseDSPCodec chess_storage(DMB) *config, uint32_t CM3Address)
{
    uint32_t configBase = (uint32_t) config;
    uint32_t configBaseCM3 = (uint32_t) config->parent.statusBufferAddress;
    return (unsigned char chess_storage(DMB) *)
    		(configBase + (CM3Address - configBaseCM3));
}

/* ----------------------------------------------------------------------------
 * Function     : handshake
 * ----------------------------------------------------------------------------
 * Description  : static method provided to ensure the CM3 and LPDSP32 are 
 *                correctly synchronised at start up 
 * Inputs       : conf : pointer to the codec configuration block, used to 
 *                exchange data as part of the handshake process 
 * Outputs      : None 
 * Assumptions  : the LPDSP32 and ARM both use the same handshake protocol 
 * ------------------------------------------------------------------------- */
static void handshake(struct _baseDSPCodec chess_storage(DMB) *conf)
{
    int ready = 0;
    while (true)
    {
        switch (conf->dspStatus)
        {
        case 0:
            ready = 1;
            conf->dspStatus = 1;
            break;
        case 1:
            break;
        case 2:
            if (ready == 1)
            {
                conf->dspStatus = 3;
                return;
            }
            break;
        default:
            ready = 0;
        }
    }
}

/* ----------------------------------------------------------------------------
 * Function     : statusUpdate
 * ----------------------------------------------------------------------------
 * Description  : Static status update method to provide feedback to the ARM 
 * Inputs       : value : four bit value which will be sent to the ARM via 
 *                the dspStatus 
 * Outputs      : None 
 * Assumptions  : None 
 * ------------------------------------------------------------------------- */
static void statusUpdate(uint32_t value)
{
    *statusLocation = (*statusLocation << 4) + (value & 0xF);
}

/* ----------------------------------------------------------------------------
 * Function     : enabled
 * ----------------------------------------------------------------------------
 * Description  : Static helper method to indicate if a flag bit is set 
 * Inputs       : action : the action being requested, which consists of a 
 *                number of request bits 
 *                flag : the specific request bit we want to check 
 * Outputs      : True if the specified flag bit in the action is set 
 * Assumptions  : None 
 * ------------------------------------------------------------------------- */
static inline bool enabled(uint8_t action, uint8_t flag)
{
    return (action & flag) == flag;
}


/* ----------------------------------------------------------------------------
 * Function     : initialiseInterruptVectors
 * ----------------------------------------------------------------------------
 * Description  : Initialise vectors to be off excpt ARM1
 * Inputs       : None
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void initialiseInterruptVectors(void)
{
	set_interrupt_mask(1 << ARM1DSP_IRQDSP);
}



/* ----------------------------------------------------------------------------
 * Function     : main
 * ----------------------------------------------------------------------------
 * Description  : The main entry point for the codec program, this sets up 
 *                the memory areas, enables the handshake between the 
 *                processors and then spins waiting for requests from the 
 *                ARM. 
 *                Note : this halts the core when not busy and is awoken by 
 *                an interrupt from the CM3 
 * Inputs       : None 
 * Outputs      : returns zero but the code is not expected to ever complete 
 * Assumptions  : None 
 * ------------------------------------------------------------------------- */
extern "C" int main(void)
{
    /* Define heap as needed (ALLOC_INIT expects words so shift the value) */
    alloc_init(MALLOC_HEAP_SIZE >> 2, 0);

    /* define the pointer to the configuration block shared between
     * the ARM and LPDSP32
     */
    struct _baseDSPCodec chess_storage(DMB) *configuration = (struct _baseDSPCodec chess_storage(DMB) *) sharedMemory;

    /* define the status update location within the config block
     * (note it is shared with the dspStatus, this is deliberate)
     */
    statusLocation = (volatile uint32_t *) &configuration->dspStatus;

    /* Initialise scratch memory if we are using it */
    scratchInitialise((void *) &sharedMemory[0x100]);
    scratchLog(0x00FFFFFF);

    /* assume nothing ready to be processed yet */
    actionRequired = false;

    /* enable the interrupts */
    initialiseInterruptVectors();
    enable_interrupts();

    /* handshake with the CM3 to get the processors in synch */
    handshake(configuration);

    /* spin forever, waiting for interrupts from the CM3 */
    while (true)
    {
        statusUpdate(4);
        core_halt();
        statusUpdate(5);

        /* only do something when responding to an interrupt from the CM3 */
        if (actionRequired == true)
        {
            actionRequired = false;
            statusUpdate(6);

            /* calculate the addresses of the incoming and outgoing buffers */
            unsigned char chess_storage(DMB) *incoming = relocateSharedAddress(configuration, configuration->parent.inputBufferAddress);
            unsigned char chess_storage(DMB) *outgoing = relocateSharedAddress(configuration, configuration->parent.outputBufferAddress);

            /* deal with whatever action is requested */
            uint8_t action = configuration->parent.operation.action;
            if (enabled(action, ENCODE_RESET))
            {
            	statusUpdate(7);
                encodeReset((pBaseDSPCodec) configuration);
            }
            if (enabled(action, ENCODE))
            {
            	statusUpdate(8);
                encode((pBaseDSPCodec) configuration, (unsigned char *) incoming, (unsigned char *) outgoing);
            }
            if (enabled(action, DECODE_RESET))
            {
            	statusUpdate(9);
                decodeReset((pBaseDSPCodec) configuration);
            }
            if (enabled(action, DECODE))
            {
            	statusUpdate(10);
                decode((pBaseDSPCodec) configuration, (unsigned char *) incoming, (unsigned char *) outgoing, enabled(action, DECODE_PLC));
            }
            if (enabled(action, CONFIGURE))
            {
            	statusUpdate(11);
                configure((pBaseDSPCodec) configuration);
            }

            /* notify the CM3 that we have completed processing */
            statusUpdate(12);
            CssCmdGen = ARM_CMD_0;
        }
    }

    return 0;
}
