# ----------------------------------------------------------------------------
# Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
# ON Semiconductor), All Rights Reserved
#
# This code is the property of ON Semiconductor and may not be redistributed
# in any form without prior written permission from ON Semiconductor.
# The terms of use and warranty for this code are covered by contractual
# agreements between ON Semiconductor and the licensee.
#
# This is Reusable Code.
#
# -----------------------------------------------------------------------------
#

from struct import unpack
from DSPSegmentKey import DSPSegmentKey

# Define base and extent of the PM, DM_High and DM_Low memory areas
#
_PM_BASE        = 0x00000000
_PM_TOP         = 0x00200000

_DM_LOW_BASE    = 0x00000000
_DM_LOW_TOP     = 0x00010000

_DM_HIGH_BASE   = 0x00800000
_DM_HIGH_TOP    = 0x00804000

# Define the segment type tags used by the LPDSP32 ELF file to identify the
# Memory Map and Memory Name segments
# 
_LPDSP32_SEGMENT_MEMORY_MAP  = 0x70123456
_LPDSP32_MEMORY_NAMES        = 0x70123457

class DSPMemoryIntegrator(object):
    '''
    This class provides the hooks to convert the memory segments stored
    in an LPDSP32 ELF file into C header and source files which can be
    incorporated into a CM3 program.
    '''
    
    def __init__(self, elf, output, tag, v1):
        '''
        Constructor
        
        elf
            An ELFFile object that has already parsed the original file into
            parsable structures
        output
            The folder to which the generated files will be written
        tag
            A tag that is used when creating file and structure names so that
            they are identifiable by the CM3
        v1
            Flag indicating of old style formatting should be handled.
        '''
        self.elf = elf
        self.output = output
        self.tag = tag
        self._unpackUint32 = ">LL" if v1 else "<LL"
        self._divisor = 3 if v1 else 1 
        self._createSegmentTypeMapping()
        
    def _uint32Generator(self, segmentData):
        ''' generate a uint32_t item from packed data ''' 
        for chunk in [segmentData[i:i+8] for i in range(0, len(segmentData), 8)]:
            yield unpack(self._unpackUint32, chunk)
        
    def _createSegmentMap(self, segmentData):
        ''' create a single segment mapping table '''
        self._segmentMap = dict(pair for pair in self._uint32Generator(segmentData))
    
    def _createSegmentTypeMapping(self):
        '''
        Construct a mapping table so we can recover the types of segments later
        '''
        self._segmentIndexMap = {}
        for index in range(self.elf.num_segments()):
            segment = self.elf.get_segment(index)
            self._segmentIndexMap[DSPSegmentKey(segment.header, segment.data())] = index
            if segment.header.p_type == _LPDSP32_MEMORY_NAMES:
                self._segmentTypes = segment.data()
            elif segment.header.p_type == _LPDSP32_SEGMENT_MEMORY_MAP:
                self._createSegmentMap(segment.data())
        
    def _segmentTypeString(self, index):
        ''' retrieve the segment type string '''
        interesting = self._segmentTypes[index:].split('\0')
        return interesting[0]
    
    def _segmentType(self, segment):
        ''' retrieve the type of LOADable segments '''
        if segment.header.p_type == 'PT_LOAD':
            segIndex = self._segmentIndexMap[DSPSegmentKey(segment.header, segment.data())]
            segTypeIndex = self._segmentMap[segIndex]
            return self._segmentTypeString(segTypeIndex)
        return None
    
    def _area(self, areaName):
        ''' helper routine defining an area name format '''
        return "%s_%s" % (self.tag, areaName)
    
    # When creating the memory overview header, we define the memoryOverview and
    # memoryOverviewEntry using conditional compilation, this may be defined in
    # multiple header files as each module is standalone. The conditional
    # compilation ensures we don't get compilation errors if more than one
    # header file is included.
    def _createMemoryOverviewHeader(self, pmTag, dmhTag, dmlTag):
        ''' Create a memory overview file '''
        define = "_%s_H_" % (self.tag)
        header = "%s/%s.h" % (self.output, self.tag)
        with open(header, "w") as output:
            output.write("#ifndef %s\n" % define.upper())
            output.write("#define %s 1\n" % define.upper())
            output.write("\n")
            output.write("/* Auto generated file\n")
            output.write(" */\n")
            output.write("\n")
            output.write("#include <stdint.h>\n")
            output.write("\n")
            output.write("#include \"%s.h\"\n" % self._area(pmTag))
            output.write("#include \"%s.h\"\n" % self._area(dmhTag))
            output.write("#include \"%s.h\"\n" % self._area(dmlTag))
            output.write("\n")
            output.write("#ifndef _MEMORY_OVERVIEW_DESCRIPTION_\n")
            output.write("#define _MEMORY_OVERVIEW_DESCRIPTION_ 1\n")
            output.write("\n")
            output.write("typedef struct {\n")
            output.write("    memoryDescription   *entries;\n")
            output.write("    uint32_t            count;\n")
            output.write("} memoryOverviewEntry;\n")                
            output.write("\n")
            output.write("typedef struct {\n")
            output.write("    memoryOverviewEntry    PM;\n")
            output.write("    memoryOverviewEntry    DMH;\n")
            output.write("    memoryOverviewEntry    DML;\n")
            output.write("} memoryOverview;\n")                
            output.write("\n")
            output.write("#endif\n")
            output.write("\n")
            output.write("extern memoryOverview __attribute__ ((section (\".dsp\"))) %s_Overview;\n" % self.tag)
            output.write("\n")
            output.write("#endif\n")

    def _findSegments(self, tag, base, top):
        '''
        find all segments of the defined type, which are contained within the
        bounds of the "base" and "top" indicators
        '''
        segments = []
        for segment in self.elf.iter_segments():
            if self._segmentType(segment) == tag:
                if (segment.header.p_vaddr >= base) and (segment.header.p_vaddr < top):
                    segments.append(DSPSegmentKey(segment.header, segment.data()))
        return segments
        
    def _overviewSegments(self, area, suffix, base, top):
        ''' helper method to provide a single segment in a standard form '''
        compositeName = area if suffix == '' else "%s_%s" % (area, suffix)
        listName = "%s_%s_SegmentList" % (self.tag, compositeName)
        segments = self._findSegments(area, base, top)
        return "    {%s, %d}" % (listName, len(segments)) if len(segments) > 0 else None
    
    def writeOverviewSegments(self, output, text):
        if text is None:
            return
        output.write("%s,\n" % text)
        
    def _createMemoryOverview(self, pm, pmSuffix, dmh, dmhSuffix, dml, dmlSuffix):
        ''' Create the memory overview structure '''
        source = "%s/%s.c" % (self.output, self.tag)
        with open(source, "w") as output:
            output.write("/* Auto generated file\n")
            output.write(" */\n")
            output.write("\n")
            output.write("#include \"%s.h\"\n" % self.tag)
            output.write("\n")
            output.write("memoryOverview %s_Overview = {\n" % self.tag)
            self.writeOverviewSegments(output, self._overviewSegments(pm, pmSuffix, _PM_BASE, _PM_TOP))
            self.writeOverviewSegments(output, self._overviewSegments(dmh, dmhSuffix, _DM_HIGH_BASE, _DM_HIGH_TOP))
            self.writeOverviewSegments(output, self._overviewSegments(dml, dmlSuffix, _DM_LOW_BASE, _DM_LOW_TOP))
            output.write("};\n")
            output.write("\n")

    def _createMemorySource(self, areaName, suffix, generator):
        '''
        Create a C source file containing descriptions of each memory section
        in the specified memory area.
        '''
        compositeName = areaName if suffix == '' else "%s_%s" % (areaName, suffix)
        taggedName = self._area(compositeName)
        source = "%s/%s.c" % (self.output, taggedName)
        header = "%s.h" % taggedName
        with open(source, "w") as output:
            output.write("/* Auto generated file\n")
            output.write(" */\n")
            output.write("\n")
            output.write("#include \"%s\"\n" % header)
            output.write("\n")
            for line in generator(areaName, taggedName):
                output.write("%s\n" % line)
            output.write("\n")
    
    # When creating the memory header, we define the memoryDescription structure
    # using conditional compilation, this may be defined in multiple header
    # files as each module is standalone. The conditional compilation ensures
    # we don't get compilation errors if more than one header file is included.
    def _createMemoryHeader(self, areaName, suffix, generator):
        '''
        Create a header file describing the contents of the C file associated
        with the given memory space.
        '''
        compositeName = areaName if suffix == '' else "%s_%s" % (areaName, suffix)
        taggedName = self._area(compositeName)
        header = "%s/%s.h" % (self.output, taggedName)
        define = "_%s_H_" % (taggedName)
        with open(header, "w") as output:
            output.write("#ifndef %s\n" % define.upper())
            output.write("#define %s 1\n" % define.upper())
            output.write("\n")
            output.write("/* Auto generated file\n")
            output.write(" */\n")
            output.write("\n")
            output.write("#include <stdint.h>\n")
            output.write("\n")
            output.write("#ifndef _MEMORY_DESCRIPTION_\n")
            output.write("#define _MEMORY_DESCRIPTION_ 1\n")
            output.write("\n")
            output.write("typedef struct {\n")
            output.write("    void        *buffer;\n")
            output.write("    uint32_t    fileSize;\n")
            output.write("    uint32_t    memSize;\n")
            output.write("    uint32_t    vAddress;\n")
            output.write("} memoryDescription;\n")                
            output.write("\n")
            output.write("#endif\n")
            output.write("\n")
            output.write("extern memoryDescription  __attribute__ ((section (\".dsp\"))) %s_SegmentList[];\n" % taggedName)
            output.write("\n")
            output.write("#endif\n")
    
    def _highByte40(self, b):
        ''' fetch high byte of 40 bit encoded program word '''
        return format(b, '#04x')
    
    def _midByte40(self, b1, b2):
        ''' fetch middle byte of 40 bit encoded program word '''
        return format((b1 & 0x0f) | ((b2 << 4) & 0xf0), '#04x')
    
    def _lowByte40(self, b1, b2):
        ''' fetch low byte of 40 bit encoded program word '''
        return format(((b1 >> 4) & 0x0f) | ((b2 << 4) & 0xf0), '#04x')
    
    def _fortyBit(self, chunk):
        ''' fetches the next program word as a encoded 40 bit value '''
        if len(chunk) != 6:
            raise ValueError("Unexpected PM Size")
        
        return "    %s, %s, %s, %s, %s," % \
            (self._highByte40(chunk[5]), self._highByte40(chunk[4]),
             self._midByte40(chunk[3], chunk[2]),
             self._lowByte40(chunk[2], chunk[1]), self._lowByte40(chunk[1], chunk[0]))
    
    def _thirtyTwoBit(self, chunk):
        '''
        Fetches the next 32 bits of data, padding it if the remaining data
        is not big enough
        '''
        while len(chunk) < 4:
            chunk.append(0)
        return ", ".join(format(b, '#04x') for b in chunk)
    
    def _listSegments(self, tag, segments, divisor=1):
        '''
        Method to provide the data for each segment in a defined format for
        the overview sections.
        '''
        if len(segments) > 0:    
            yield "memoryDescription  __attribute__ ((section (\".dsp\"))) %s_SegmentList[] = {" % tag
            for segment in segments:
                yield "    {%s_%s, 0x%08x, 0x%08x, 0x%08x}," % (tag, segment.vaddr, segment.filesz, segment.memsz, segment.vaddr / divisor )
            yield "};\n"
        
    def _PMGenerator(self, areaName, tag):
        '''
        Generator for a given program memory segment
        
        This will create the initial data array for each provided segment and
        then an overview table which provides details of each memory area
        in the file and how to access it.
        '''        
        segments = self._findSegments(areaName, _PM_BASE, _PM_TOP)
        for segment in segments:
            yield "uint8_t  __attribute__ ((section (\".dsp\"))) %s_%s[] = {" % (tag, segment.vaddr)
            data = bytearray(segment.data)
            for chunk in [data[i:i+6] for i in range(0, len(data), 6)]:
                yield self._fortyBit(chunk)
            yield "};\n"
            
        for line in self._listSegments(tag, segments, self._divisor):
            yield line 
        
    def _paddedAndAligned(self, data, address):
        '''
        This ensures that each data area is correctly aligned and padded to full
        word boundaries. This is necessary to ensure the Flash Copier can be
        effectively used by the CM3 when loading the data area.
        '''
        result = bytearray(data)
        
        # If we don't start on a word boundary, insert bytes at the start
        bytesToAdd = address & 3
        while (bytesToAdd > 0):
            result.insert(0, 0)
            bytesToAdd -= 1

        # if we don't end on a word boundary, append bytes at the end
        while (len(result) & 3) > 0:
            result.append(0)        
                
        return result
        
    def _DMGenerator(self, tag, segments):
        '''
        Generator for a given data memory segment
        
        This will create the initial data array for each provided segment and
        then an overview table which provides details of each memory area
        in the file and how to access it.
        '''
        # Process each segment in turn
        for segment in segments:
            yield "uint8_t  __attribute__ ((section (\".dsp\"))) %s_%s[] = {" % (tag, segment.vaddr)
            data = self._paddedAndAligned(segment.data, segment.vaddr)
            for chunk in [data[i:i+4] for i in range(0, len(data), 4)]:
                yield "    %s," % self._thirtyTwoBit(chunk)
            yield "};\n"
                    
        # Create the memory area overview for this type of memory 
        for line in self._listSegments(tag, segments):
            yield line 
        
    def _DMLowGenerator(self, areaName, tag):
        '''
        Generator which provides the segment details for the high data memory 
        '''
        segments = self._findSegments(areaName, _DM_LOW_BASE, _DM_LOW_TOP)
        for line in self._DMGenerator(tag, segments):
            yield line 
        
    def _DMHighGenerator(self, areaName, tag):
        '''
        Generator which provides the segment details for the high data memory 
        '''
        segments = self._findSegments(areaName, _DM_HIGH_BASE, _DM_HIGH_TOP)
        for line in self._DMGenerator(tag, segments):
            yield line 
    
    def _createMemoryFiles(self, areaName, suffix, generator):
        '''
        Create the source and header files associated with a single memory area
        '''
        self._createMemorySource(areaName, suffix, generator)
        self._createMemoryHeader(areaName, suffix, generator)
    
    def parseMemoryAreas(self):
        '''
        This is the only public method in this class and it manages the
        extraction process for each of the memory areas and then creates the
        overview files
        '''
        self._createMemoryFiles('PM', '', self._PMGenerator)
        self._createMemoryFiles('DM', 'Hi', self._DMHighGenerator)
        self._createMemoryFiles('DM', 'Lo', self._DMLowGenerator)
        
        self._createMemoryOverviewHeader('PM', 'DM_Hi', 'DM_Lo')
        self._createMemoryOverview('PM', '', 'DM', 'Hi', 'DM', 'Lo')
