/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ----------------------------------------------------------------------------
 * main.c
 * - Simple application to configure the LPDSP32 JTAG port using DIOs
 * ----------------------------------------------------------------------------
 * $Revision: 1.22 $
 * $Date: 2017/08/14 13:07:32 $
 * ------------------------------------------------------------------------- */
#include <rsl10.h>

#define CONCAT(x, y)                    x##y
#define DIO_SRC(x)                      CONCAT(DIO_SRC_DIO_, x)

/* define the DIOs required for configuring the LPDSP32 JTAG connection */
#define JTAG_LPDSP32_TCK	(1)
#define JTAG_LPDSP32_TDI	(2)
#define JTAG_LPDSP32_TDO	(3)
#define JTAG_LPDSP32_TMS	(4)

/* DIO number that is connected to push button of EVB */
#define BUTTON_DIO      	5

/* DIO number that is connected to LED of EVB */
#define LED_DIO             (6)

/* DIO number that is used for easy re-flashing (recovery mode) */
#define RECOVERY_DIO		(12)

/* We need to maintain some state information to know when we are processing */
typedef enum {
	stateIDLE,
	stateFIRE,
	stateBUSY,
	stateDONE
} status_t;

/* Keep a flag so we know if we should allow a number to be calculated */
volatile status_t ourState = stateIDLE;

/* ----------------------------------------------------------------------------
 * Function      : void DIO0_IRQHandler(void)
 * ----------------------------------------------------------------------------
 * Description   : If we are IDLE, then fire an event to get the next value
 *                 from the LPDSP32
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void DIO0_IRQHandler(void)
{
    static uint8_t ignore_next_dio_int = 0;
    if (ignore_next_dio_int)
    {
        ignore_next_dio_int = 0;
    }
    else if ((DIO_DATA->ALIAS[BUTTON_DIO] == 0) && (ourState == stateIDLE))
    {
        /* Button is pressed: Ignore next interrupt.
         * This is required to deal with the debounce circuit limitations. */
        ignore_next_dio_int = 1;

        /* set the state to FIRE, indicates something needs to be done */
        ourState = stateFIRE;
    }
}

/* ----------------------------------------------------------------------------
 * Function : DSP0_IRQHandler
 * ----------------------------------------------------------------------------
 * Description : Interrupt service handler which is activated when the LPDSP32
 * has completed it processing of the data
 *
 * Inputs :
 * 		None
 *
 * Outputs :
 * 		No specific output but will set the state to DONE.
 *
 * Assumptions : None
 * ----------------------------------------------------------------------------
 */
void DSP0_IRQHandler(void) {
	if (ourState == stateBUSY)
		ourState = stateDONE;
}

/* ----------------------------------------------------------------------------
 * Function      : void Initialize(void)
 * ----------------------------------------------------------------------------
 * Description   : Initialize the system by disabling interrupts, configuring
 *                 the required DIOs and DIO interrupt,
 *                 updating SystemCoreClockUpdate and enabling interrupts.
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void Initialize(void)
{
    /* Mask all interrupts */
    __disable_irq();

    /* Disable all existing interrupts, clearing all pending source */
    Sys_NVIC_DisableAllInt();
    Sys_NVIC_ClearAllPendingInt();

    /* Test DIO12 to pause the program to make it easy to re-flash */
    DIO->CFG[RECOVERY_DIO] = DIO_MODE_INPUT  | DIO_WEAK_PULL_UP |
                             DIO_LPF_DISABLE | DIO_6X_DRIVE;
    while (DIO_DATA->ALIAS[RECOVERY_DIO] == 0);

    /* Enable output on the LED */
    Sys_DIO_Config(LED_DIO, DIO_MODE_GPIO_OUT_0);

    /* Setup DIO5 as a GPIO input with interrupts on transitions. */
    Sys_DIO_Config(BUTTON_DIO, DIO_MODE_GPIO_IN_0 | DIO_WEAK_PULL_UP |
                   DIO_LPF_DISABLE);
    Sys_DIO_IntConfig(0,
                      DIO_EVENT_TRANSITION | DIO_SRC(BUTTON_DIO) |
                      DIO_DEBOUNCE_ENABLE,
                      DIO_DEBOUNCE_SLOWCLK_DIV1024, 49);
    NVIC_EnableIRQ(DIO0_IRQn);

    /* Enable the interrupt zero from the LPDSP32 */
    NVIC_EnableIRQ(DSP0_IRQn);

    /* Unmask all interrupts */
    __enable_irq();
}

/* ----------------------------------------------------------------------------
 * Function      : int main(void)
 * ----------------------------------------------------------------------------
 * Description   : Initialize the system, then toggle DIO6 as controlled by
 *                 DIO5 (press to toggle input/output).
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
int main(void)
{
	uint32_t *sharedMemory = (uint32_t *) 0x20011800;

	uint32_t sequenceNumber = 0;
	uint32_t fibonacciValue = 0;

    /* Initialize the system */
    Initialize();

    /* Enable the JTAG for the LPDSP32 using the DIOs we defined above */
	Sys_LPDSP32_DIOJTAG(DIO_WEAK_PULL_DOWN, JTAG_LPDSP32_TDI,
			JTAG_LPDSP32_TMS, JTAG_LPDSP32_TCK, JTAG_LPDSP32_TDO);

	/* Set the LED off until a button is pressed */
    Sys_GPIO_Set_Low(LED_DIO);

    /* Spin loop */
    while (1)
    {
    	switch (ourState) {

    	case stateIDLE:
    		/* don't need to do anything, so just refresh the watchdog and
    		 * wait for some time */
            Sys_Watchdog_Refresh();
    		break;

    	case stateFIRE:
    		/* Button has been pressed, tell the LPDSP32 to do something */
    	    Sys_GPIO_Set_High(LED_DIO);
    	    ourState = stateBUSY;

    	    /* set up the next sequence number and set the result to some known
    	     * bad result */
    	    *(sharedMemory) = sequenceNumber++;
    	    *(sharedMemory + 1) = 0x00000BAD;
    	    if (sequenceNumber > 48)
    	    	sequenceNumber = 0;

    	    // Tell the DSP there is data available to deal with.
    	    SYSCTRL_DSS_CMD->DSS_CMD_1_ALIAS = DSS_CMD_1_BITBAND;
    		break;

    	case stateBUSY:
    		/* We will remain in the busy state until we receive the interrupt
    		 * from the LPDSP32 indicating it has completed the calculation,
    		 * the ISR will handle changing the state to DONE.
    		 */
    		break;

    	case stateDONE:
    		/* LPDSP32 has responded, turn off the LED and revert to IDLE */
    	    Sys_GPIO_Set_Low(LED_DIO);
    	    fibonacciValue = sharedMemory[1];
    	    ourState = stateIDLE;
    		break;
    	}

    	Sys_Delay_ProgramROM(0.1 * SystemCoreClock);
    }
}
